<?php

namespace Drupal\ckeditor5_wordcount\Form;

use Drupal\Core\Form\ConfigFormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Asset\LibraryDiscoveryInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Configure CKEditor 5 Word Count settings for this site.
 */
class WordCountConfigForm extends ConfigFormBase {

  /**
   * The library discovery service.
   *
   * @var \Drupal\Core\Asset\LibraryDiscoveryInterface
   */
  protected $libraryDiscovery;

  /**
   * Constructs a WordCountConfigForm object.
   *
   * @param \Drupal\Core\Asset\LibraryDiscoveryInterface $library_discovery
   *   The library discovery service.
   */
  public function __construct(LibraryDiscoveryInterface $library_discovery) {
    $this->libraryDiscovery = $library_discovery;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('library.discovery')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'ckeditor5_wordcount_config';
  }

  /**
   * {@inheritdoc}
   */
  protected function getEditableConfigNames() {
    return ['ckeditor5_wordcount.settings'];
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state) {
    $config = $this->config('ckeditor5_wordcount.settings');

    $form['limits'] = [
      '#type' => 'fieldset',
      '#title' => $this->t('Content Limits'),
      '#description' => $this->t('Set limits for word and character counts. When exceeded, the editor background will turn light red.'),
    ];

    $form['limits']['enable_word_limit'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Enable word limit'),
      '#default_value' => $config->get('enable_word_limit') ?? FALSE,
    ];

    $form['limits']['word_limit'] = [
      '#type' => 'number',
      '#title' => $this->t('Word limit'),
      '#description' => $this->t('Maximum number of words allowed. Set to 0 for no limit.'),
      '#default_value' => $config->get('word_limit') ?? 500,
      '#min' => 0,
      '#states' => [
        'visible' => [
          ':input[name="enable_word_limit"]' => ['checked' => TRUE],
        ],
      ],
    ];

    $form['limits']['enable_character_limit'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Enable character limit'),
      '#default_value' => $config->get('enable_character_limit') ?? FALSE,
    ];

    $form['limits']['character_limit'] = [
      '#type' => 'number',
      '#title' => $this->t('Character limit'),
      '#description' => $this->t('Maximum number of characters allowed. Set to 0 for no limit.'),
      '#default_value' => $config->get('character_limit') ?? 2000,
      '#min' => 0,
      '#states' => [
        'visible' => [
          ':input[name="enable_character_limit"]' => ['checked' => TRUE],
        ],
      ],
    ];

    $form['appearance'] = [
      '#type' => 'fieldset',
      '#title' => $this->t('Appearance'),
    ];

    $form['appearance']['warning_threshold'] = [
      '#type' => 'number',
      '#title' => $this->t('Warning threshold (%)'),
      '#description' => $this->t('Show warning when this percentage of the limit is reached. Set to 100 to only show error state.'),
      '#default_value' => $config->get('warning_threshold') ?? 90,
      '#min' => 1,
      '#max' => 100,
      '#field_suffix' => '%',
    ];

    return parent::buildForm($form, $form_state);
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    $this->config('ckeditor5_wordcount.settings')
      ->set('enable_word_limit', $form_state->getValue('enable_word_limit'))
      ->set('word_limit', $form_state->getValue('word_limit'))
      ->set('enable_character_limit', $form_state->getValue('enable_character_limit'))
      ->set('character_limit', $form_state->getValue('character_limit'))
      ->set('warning_threshold', $form_state->getValue('warning_threshold'))
      ->save();

    $this->libraryDiscovery->clearCachedDefinitions();
    parent::submitForm($form, $form_state);
  }

}
